const electron = require('electron')
const emenu = electron.Menu
const app = electron.app;
const dialog = electron.dialog
const fs = require('fs')
const isDev = require('electron-is-dev');

var devView = [
		{label: 'Reload', accelerator: 'CmdOrCtrl+R', 
		 click (item, focusedWindow) {if (focusedWindow) focusedWindow.reload()}
		},
		{label: 'Toggle Developer Tools', accelerator: process.platform === 'darwin' ? 'Alt+Command+I' : 'Ctrl+Shift+I',
		 click (item, focusedWindow) {if (focusedWindow) focusedWindow.webContents.toggleDevTools()}
			},
		{type: 'separator'},
		{label: 'Toggle Full Screen', accelerator: (function () {if (process.platform === 'darwin') {return 'Ctrl+Command+F'} else {return 'F11' }})(),
     click: function (item, focusedWindow) {if (focusedWindow) {focusedWindow.setFullScreen(!focusedWindow.isFullScreen())}}
  	}, 
	]


var editmenu = 	{
			label: 'Edit',
			submenu: [
				{role:'copy'},
				{role:'paste'}
			]
		}
		
var prodView = [
		{label: 'Toggle Full Screen', accelerator: (function () {if (process.platform === 'darwin') {return 'Ctrl+Command+F'} else {return 'F11' }})(),
     click: function (item, focusedWindow) {if (focusedWindow) {focusedWindow.setFullScreen(!focusedWindow.isFullScreen())}}
  	}, 
	]
	
var template = [
		{
			label: 'Process',
			submenu: [
				{label: 'Open File',  accelerator: 'CmdOrCtrl+O', click: function (item, bw, e) {handleFileMenu(item, bw, e)}, id:"open"},
				{type: 'separator'}, 
				{label: 'Modes',
					submenu: [
							{label: 'Colored Image', type: 'checkbox', checked: true, click: function (item, bw, e) {handleModesMenu(item, bw, e)}, id:"color"},
							{label: 'No Background', type: 'checkbox', checked: true, click: function (item, bw, e) {handleModesMenu(item, bw, e)}, id:"normal"},
							{label: 'Hairline', type: 'checkbox', checked: false, click: function (item, bw, e) {handleModesMenu(item, bw, e)}, id:"hairline"},
							{label: 'Outlines', type: 'checkbox', checked: false, click: function (item, bw, e) {handleModesMenu(item, bw, e)}, id:"outline"}
					//	, {label: 'Outside Edge', type: 'checkbox', checked: false, click: function (item, bw, e) {handleModesMenu(item, bw, e)}, id:"outside"}
						]
				}
			]
		},
		{
			label: 'View',
			submenu: []
		},
		{
			label: 'Window',
			role: 'window',
			submenu: [
				{label: 'Minimize', accelerator: 'CmdOrCtrl+M',role: 'minimize'},
				{label: 'Close', accelerator: 'CmdOrCtrl+W',role: 'close'}, 
				{type: 'separator'}, 
				{label: 'Reopen Window',accelerator: 'CmdOrCtrl+Shift+T', enabled: false, key: 'reopenMenuItem',
					click: function () {app.emit('activate')}
			}]
		}, 
		{
			label: 'Help',
			role: 'help',
			submenu: [{
				label: 'Learn More',
				click: function () {
				//	electron.shell.openExternal('http://electron.atom.io')
				}
			}]
		}]

template[1].submenu =  isDev ? devView : prodView

if (process.platform === 'darwin') {
  template.unshift({
    label: app.getName(),
    submenu: [
      {role: 'about'},
      {type: 'separator'},
      {role: 'services', submenu: []},
      {type: 'separator'},
      {role: 'hide'},
      {role: 'hideothers'},
      {role: 'unhide'},
      {type: 'separator'},
      {role: 'quit'}
    ]
  })

  // Window menu
  template[3].submenu = [
    {role: 'close'},
    {role: 'minimize'},
    {role: 'zoom'},
    {type: 'separator'},
    {role: 'front'}
  ]
}

if (isDev) template.splice(2,0, editmenu);


	
	function handleFileMenu (item, browserWindow, event){
		if (! dialog) return;
		switch(item.id){
			case 'open': openProject(browserWindow); break;
			break;
		}
	}

	function handleModesMenu (item, bw, event){setProcessMode(bw,item.id);}
	
	function  openProject (bw){
		var files  = dialog.showOpenDialog({properties: ['openFile']})
		if (!files || files.length == 0) return;
		var name = files[0];
		bw.webContents.send('main-msg', ["open", name])
	}


saveProject = function (bw, name, str){
		if (name) fs.writeFile(name, str, fileWriteResult);
		function fileWriteResult (err) {
			if(err) {
				bw.webContents.send('main-msg', ["saveerror", err]);
				return console.log(err);
			}
			else bw.webContents.send('main-msg', ["saved"]);
		}
	}

setProcessMode = function (bw, mode){
		var id = mode;	
		var appmenu = emenu.getApplicationMenu();
		var choices = appmenu.items[1].submenu.items[2].submenu.items;
	
		for (var pos in choices){
			var sm = choices[pos];
			if (sm.id  == mode) { // overrride default behaviour
				sm.checked = true;
				id = sm.id;
			}
			else 	sm.checked = false;
		}
		bw.webContents.send('main-msg', ["changeMode", id]);
	}

const menu = emenu.buildFromTemplate(template)
emenu.setApplicationMenu(menu)

exports.save = saveProject;
exports.setMode = setProcessMode;